import { parse, serialize } from "cookie";
import { BagelUsersRequest, BagelDB } from "@bageldb/bagel-db/src/server";
import * as _axios from "axios";
const AUTH_ENDPOINT = "https://auth.bageldb.com/api/public";
const axios = _axios.default;
class BagelNuxtUser extends BagelUsersRequest {
  constructor({ instance }) {
    super({ instance });
    this.ctx = this.instance.ctx;
  }
  async _bagelUserActive() {
    const userId = await this.getBagelUserID();
    return userId !== null && userId?.length > 0;
  }
  getUser() {
    return new Promise(async (resolve, reject) => {
      if (!this._bagelUserActive()) {
        reject(new Error("a Bagel User must be logged in to get Bagel User info"));
        return;
      }
      const url = `${AUTH_ENDPOINT}/user`;
      await this.instance.axiosInstance.get(url).then(
        (res) => {
          if (res.status == 200) {
            resolve(res);
          } else {
            reject(res);
          }
        }
      ).catch(
        (err) => {
          reject(err);
        }
      );
    });
  }
  _isBrowser() {
    const isNode = new Function("try {return this===global;}catch(e){return false;}");
    return !isNode();
  }
  _getCookie(key) {
    let cookieStr;
    if (!process.client && this.ctx.req) {
      cookieStr = this.ctx.req.headers.cookie;
    } else if (process.client) {
      cookieStr = document.cookie;
    }
    const cookies = parse(cookieStr || "") || {};
    return cookies[key];
  }
  _setCookie(key, value, options = {}) {
    if (process.server && !this.ctx.res)
      return;
    const _key = key;
    const _value = typeof value === "string" ? value : JSON.stringify(value);
    const _options = options || {};
    _options.expires = new Date(Date.now() + 7 * 864e5);
    const serializedCookie = serialize(_key, _value, _options);
    if (process.client) {
      document.cookie = serializedCookie;
    } else if (process.server && this.ctx.res) {
      const cookies = this.ctx.res.getHeader("Set-Cookie") || [];
      cookies.unshift(serializedCookie);
      this.ctx.res.setHeader(
        "Set-Cookie",
        cookies.filter(
          (v, i, arr) => arr.findIndex((val) => val.startsWith(v.substr(0, v.indexOf("=")))) === i
        )
      );
    }
    return _value;
  }
  _removeCookie(key) {
    this._setCookie(key, void 0, { maxAge: -1 });
  }
  _storeBagelUser(userID) {
    this._setCookie("bagel-user", userID);
  }
  async _storeTokens(data) {
    if (data?.access_token) {
      this._setCookie("bagel-access", data.access_token);
    }
    if (data?.expires_in) {
      const expires = new Date(Date.now() + data.expires_in * 1e3);
      this._setCookie("bagel-expires", expires.toISOString());
    }
    if (data?.refresh_token) {
      this._setCookie("bagel-refresh", data.refresh_token);
    }
  }
  async getBagelUserID() {
    return this._getCookie("bagel-user");
  }
  async _getRefreshToken() {
    return this._getCookie("bagel-refresh");
  }
  async _getAccessToken() {
    return this._getCookie("bagel-access");
  }
  async logout() {
    this._removeCookie("bagel-user");
    this._removeCookie("bagel-user");
    this._removeCookie("bagel-refresh");
  }
}
export default class BagelNuxt extends BagelDB {
  constructor(apiToken, ctx) {
    super(apiToken);
    this.ctx = ctx;
    this.axiosInstance = axios.create();
    this.axiosInstance.interceptors.request.use(
      async (config) => {
        if (!config.headers) {
          config.headers = {};
        }
        config.headers["Accept-Version"] = "v1";
        const userIsActive = await new BagelNuxtUser({ instance: this })._bagelUserActive();
        if (userIsActive && !config.url?.includes("user/token")) {
          const accessToken = new BagelNuxtUser({ instance: this })._getAccessToken();
          config.headers.Authorization = "Bearer " + accessToken;
        } else {
          config.headers.Authorization = "Bearer " + apiToken;
        }
        return config;
      },
      (error) => {
        return Promise.reject(error);
      }
    );
    this.axiosInstance.interceptors.response.use(
      (response) => {
        return response;
      },
      async (error) => {
        const userIsActive = await new BagelNuxtUser({ instance: this })._bagelUserActive();
        if (userIsActive && error.response && error.response.status == 401 && !error.config.url.includes("user/token")) {
          return new BagelNuxtUser({ instance: this }).refresh().then(() => {
            const config = error.config;
            config.headers["Authorization"] = `Bearer ${new BagelNuxtUser({ instance: this })._getAccessToken()}`;
            return new Promise((resolve, reject) => {
              axios.request(config).then((response) => {
                resolve(response);
              }).catch((error2) => {
                reject(error2);
              });
            });
          }).catch(() => {
            return Promise.reject(error);
          });
        }
        return Promise.reject(error);
      }
    );
  }
  users() {
    return new BagelNuxtUser({ instance: this });
  }
}
